package zeichnenohneserialisierung;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author
 * @version 1.0
 */

import java.util.Vector;

public class ZeichnenOhneSerialisierung extends Applet{
    ZeichnenPanel panel;
    ZeichnenControls controls;

    public void init() {
	setLayout(new BorderLayout());
	panel = new ZeichnenPanel();
        controls = new ZeichnenControls(panel);
	add("Center", panel);
	add("South",controls);
    }

    public void destroy() {
        remove(panel);
        remove(controls);
    }

    public static void main(String args[]) {
	Frame f = new Frame("Zeichnen");
	ZeichnenOhneSerialisierung zeichnenTest = new ZeichnenOhneSerialisierung();
	zeichnenTest.init();
	zeichnenTest.start();

	f.add("Center", zeichnenTest);
	f.setSize(300, 300);
	f.show();
    }
    public String getAppletInfo() {
        return "Einfaches Zeichenprogramm.";
    }
}

class ZeichnenPanel extends Panel implements MouseListener, MouseMotionListener {
    // Kodierung des Zeichenmodus
    public static final int LINIEN = 0;
    public static final int PUNKTE = 1;
    int	   zeichnungsModus = LINIEN;
    Vector linien = new Vector();
    Vector farben = new Vector();
    int x1,y1;
    int x2,y2;

    public ZeichnenPanel() {
	setBackground(Color.white);
	addMouseMotionListener(this);
	addMouseListener(this);
    }

    public void setzenDesZeichnungsModus(int zeichnungsModus) {
	switch (zeichnungsModus) {
	  case LINIEN:
	  case PUNKTE:
	    this.zeichnungsModus = zeichnungsModus;
	    break;
	  default:
	    throw new IllegalArgumentException();
	}
    }


    public void mouseDragged(MouseEvent e) {
        // Event Tracking : ignoriere das MausDragging
        e.consume();
        switch (zeichnungsModus) {
            case LINIEN:
                x2 = e.getX();
                y2 = e.getY();
                break;
            case PUNKTE:
            default:
                farben.addElement(getForeground());
                linien.addElement(new Rectangle(x1, y1, e.getX(), e.getY()));
                x1 = e.getX();
                y1 = e.getY();
                break;
        }
        repaint();
    }

    public void mouseMoved(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        // Event Tracking : ignoriere MousePressed
        e.consume();
        switch (zeichnungsModus) {
            case LINIEN:
                x1 = e.getX();
                y1 = e.getY();
                x2 = -1;
                break;
            case PUNKTE:
            default:
                farben.addElement(getForeground());
                linien.addElement(new Rectangle(e.getX(), e.getY(), -1, -1));
                x1 = e.getX();
                y1 = e.getY();
                repaint();
                break;
        }
    }

    public void mouseReleased(MouseEvent e) {
      // Event Tracking
        e.consume();
        switch (zeichnungsModus) {
            case LINIEN:
                farben.addElement(getForeground());
                linien.addElement(new Rectangle(x1, y1, e.getX(), e.getY()));
                x2 = -1;
                break;
            case PUNKTE:
            default:
                break;
        }
        repaint();
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void paint(Graphics g) {
	int np = linien.size();

	/* aktuelle Linie zeichnen */
	g.setColor(getForeground());
	for (int i=0; i < np; i++) {
	    Rectangle p = (Rectangle)linien.elementAt(i);
	    g.setColor((Color)farben.elementAt(i));
	    if (p.width != -1) {
		g.drawLine(p.x, p.y, p.width, p.height);
	    } else {
		g.drawLine(p.x, p.y, p.x, p.y);
	    }
	}
	if (zeichnungsModus == LINIEN) {
	    g.setColor(getForeground());
	    if (x2 != -1) {
                g.drawLine(x1, y1, x2, y2);
	    }
	}
    }
}


class ZeichnenControls extends Panel implements ItemListener {
    ZeichnenPanel target;

    public ZeichnenControls(ZeichnenPanel target) {
	this.target = target;
	setLayout(new FlowLayout());
	setBackground(Color.lightGray);
	target.setForeground(Color.red);
	CheckboxGroup group = new CheckboxGroup();
	Checkbox b;
	add(b = new Checkbox(null, group, false));
	b.addItemListener(this);
	b.setForeground(Color.red);
	add(b = new Checkbox(null, group, false));
	b.addItemListener(this);
	b.setForeground(Color.green);
	add(b = new Checkbox(null, group, false));
	b.addItemListener(this);
	b.setForeground(Color.blue);
	add(b = new Checkbox(null, group, false));
	b.addItemListener(this);
	b.setForeground(Color.pink);
	add(b = new Checkbox(null, group, false));
	b.addItemListener(this);
	b.setForeground(Color.orange);
	add(b = new Checkbox(null, group, true));
	b.addItemListener(this);
	b.setForeground(Color.black);
	target.setForeground(b.getForeground());
	Choice shapes = new Choice();
	shapes.addItemListener(this);
	shapes.addItem("Liniengrafik");
	shapes.addItem("Punktgrafik");
	shapes.setBackground(Color.lightGray);
	add(shapes);
    }

    public void paint(Graphics g) {
	Rectangle r = getBounds();
	g.setColor(Color.lightGray);
	g.draw3DRect(0, 0, r.width, r.height, false);

        int n = getComponentCount();
        for(int i=0; i<n; i++) {
            Component comp = getComponent(i);
            if (comp instanceof Checkbox) {
                Point loc = comp.getLocation();
                Dimension d = comp.getSize();
                g.setColor(comp.getForeground());
                g.drawRect(loc.x-1, loc.y-1, d.width+1, d.height+1);
            }
        }
    }

  public void itemStateChanged(ItemEvent e) {
    if (e.getSource() instanceof Checkbox) {
      target.setForeground(((Component)e.getSource()).getForeground());
    } else if (e.getSource() instanceof Choice) {
      String choice = (String) e.getItem();
      if (choice.equals("Liniengrafik")) {
	target.setzenDesZeichnungsModus(ZeichnenPanel.LINIEN);
      } else if (choice.equals("Punktgrafik")) {
	target.setzenDesZeichnungsModus(ZeichnenPanel.PUNKTE);
      }
    }
  }
}
